package com.smppw.modaq.application.components.report.parser.pdf;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.smppw.modaq.application.components.CustomPDFTextStripper;
import com.smppw.modaq.application.components.report.parser.AbstractReportParser;
import com.smppw.modaq.common.conts.Constants;
import com.smppw.modaq.common.enums.ReportParseStatus;
import com.smppw.modaq.common.enums.ReportType;
import com.smppw.modaq.common.exception.ReportParseException;
import com.smppw.modaq.domain.dto.report.ReportBaseInfoDTO;
import com.smppw.modaq.domain.dto.report.ReportData;
import com.smppw.modaq.domain.dto.report.ReportFundInfoDTO;
import com.smppw.modaq.domain.dto.report.ReportParserParams;
import com.smppw.modaq.domain.mapper.EmailFieldMappingMapper;
import org.apache.pdfbox.Loader;
import org.apache.pdfbox.io.RandomAccessReadBufferedFile;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.springframework.beans.factory.annotation.Value;
import technology.tabula.CustomObjectExtractor;
import technology.tabula.Page;
import technology.tabula.PageIterator;
import technology.tabula.Table;
import technology.tabula.extractors.SpreadsheetExtractionAlgorithm;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2024/9/29 16:45
 * @description pdf格式的报告解析抽象类
 */
public abstract class AbstractPDReportParser<T extends ReportData> extends AbstractReportParser<T> {
    /**
     * 去除了水印的所有文本内容
     */
    protected List<String> textList;

    @Value("${email.report.ai-parser-url}")
    private String aiParserUrl;

    protected String aiFileId;

    protected String aiParserContent;

    protected Boolean aiParse = false;

    public AbstractPDReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public T parse(ReportParserParams params) throws IOException, ReportParseException {
        // 初始化
        this.init();
        String filename = params.getFilename();
        String filepath = params.getFilepath();
        // 解析报告和表格
        try (PDDocument document = Loader.loadPDF(new RandomAccessReadBufferedFile(filepath))) {
            // 识别所有文字（去水印后的）
            CustomPDFTextStripper stripper = new CustomPDFTextStripper();
            stripper.setSortByPosition(true);
            String text = stripper.getText(document).replace(Constants.WATERMARK_REPLACE, StrUtil.EMPTY);
            this.textList = StrUtil.split(text, System.lineSeparator());
            this.textList.removeIf(StrUtil::isBlank);
            if (this.textList.isEmpty()) {
                throw new ReportParseException(ReportParseStatus.REPORT_IS_SCAN, filename);
            }
            // 解析所有表格（单元格字符去水印）
            List<Table> tables = ListUtil.list(true);
            SpreadsheetExtractionAlgorithm spreadsheetExtractionAlgorithm = new SpreadsheetExtractionAlgorithm();
            // 自定义表格提取工具，去除单元格中的水印文字
            PageIterator pageIterator = new CustomObjectExtractor(document).extract();
            // 确认单只解析第一页
            int i = 0;
            while (pageIterator.hasNext()) {
                Page page = pageIterator.next();
                List<Table> tableList = spreadsheetExtractionAlgorithm.extract(page);
                if (i >= 1 && params.getReportType() == ReportType.LETTER) {
                    break;
                }
                Integer rows = tableList.stream().map(Table::getRowCount).filter(rowCount -> rowCount >= 1).reduce(0, Integer::sum);
                if (rows >= 1) {
                    for (Table table : tableList) {
                        int rowCount = table.getRowCount();
                        if (rowCount >= 1) {
                            tables.add(table);
                        }
                    }
                } else if (params.getReportType() == ReportType.LETTER) {
                    this.aiParse = true;
                    Map<String, Object> paramsMap = MapUtil.newHashMap(4);
                    paramsMap.put("filepath", filepath);
                    paramsMap.put("file_id", params.getAiFileId());
                    String body = null;
                    try {
                        body = HttpUtil.get(this.aiParserUrl, paramsMap);
                        JSONObject jsonObject = JSONUtil.parseObj(body);
                        this.aiFileId = MapUtil.getStr(jsonObject, "file_id");
                        String content = StrUtil.split(jsonObject.getStr("content"), "```").get(1);
                        this.aiParserContent = "{" + StrUtil.subAfter(content, "{", false) + "}";
                    } catch (Exception e) {
                        this.logger.warn("{} ai解析失败，解析结果{}，错误原因：{}", filename, body, ExceptionUtil.stacktraceToString(e));
                    }
                }
                i++;
            }
            if (tables.isEmpty() && StrUtil.isBlank(this.aiParserContent)) {
                throw new ReportParseException(ReportParseStatus.REPORT_IS_SCAN, filename);
            }
            this.initTableInfo(tables);
        }
        try {
            // 报告基本信息
            ReportBaseInfoDTO reportInfo = this.buildReportInfo(params);
            // 解析报告中主体基金的基本信息
            ReportFundInfoDTO reportFundInfo = this.buildFundInfo(params);
            // 解析其他表格信息并且设置结果字段
            T reportData = this.parseExtInfoAndSetData(reportInfo, reportFundInfo);
            // 数据清洗后返回
            this.cleaningReportData(reportData);
            return reportData;
        } catch (ReportParseException e) {
            throw e;
        } catch (Exception e) {
            throw new ReportParseException(ReportParseStatus.NOT_A_FIXED_FORMAT, filename);
        }
    }

    /**
     * 初始化解析所有表格数据
     *
     * @param tables 按固定的表格模式划分到不同的对象中
     */
    protected abstract void initTableInfo(List<Table> tables);

    /**
     * 绑定基金基本信息
     *
     * @param params /
     * @return /
     */
    protected abstract ReportFundInfoDTO buildFundInfo(ReportParserParams params);

    /**
     * 解析报告的其他信息并设置到对象中
     *
     * @param reportInfo 报告基本信息
     * @param fundInfo   报告中基金基本信息
     * @return /
     */
    protected abstract T parseExtInfoAndSetData(ReportBaseInfoDTO reportInfo, ReportFundInfoDTO fundInfo);

    @Override
    protected void cleaningReportData(T reportData) {
        // cleaning.
    }

    @Override
    protected void init() {
        super.init();
        // 先初始化为null
        this.textList = null;
        this.aiFileId = null;
        this.aiParserContent = null;
        this.aiParse = false;
    }
}
